# mount.py
# Copyright (C) 2008 Stefan J. Betz <stefan_betz@gmx.net>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import pygtk
pygtk.require("2.0")
import gtk
import gtk.glade
import threading
import config
import string
import os
import re
import pyneighborhood

class MountWindow(threading.Thread):
    """
    The Mount Dialog
    """

    def __init__(self, mainwindow, host, share, iter):
        """
        Constructor
        """
        threading.Thread.__init__(self)
        self.mainwindow = mainwindow
        self.host = host
        self.share = share
        self.iter = iter
        self.xml = gtk.glade.XML(pyneighborhood.config.sharepath() + "/pyNeighborhood.glade", "mount")
        self.xml.signal_autoconnect(self)
        self.window = self.xml.get_widget("mount")
        self.workgroupentry = self.xml.get_widget("workgroup-entry")
        self.workgroupentry.set_text(host.workgroup)
        self.hostentry = self.xml.get_widget("host-entry")
        self.hostentry.set_text(host.name)
        self.shareentry = self.xml.get_widget("share-entry")
        self.shareentry.set_text(share)
        self.type = self.xml.get_widget("type-combo")
        self.anontoggle = self.xml.get_widget("anonymous-button")
        self.authtoggle = self.xml.get_widget("user-pass-button")
        self.user = self.xml.get_widget("user-entry")
        self.user.set_text(pyneighborhood.userconfig.get("Main", "username"))
        self.password = self.xml.get_widget("password-entry")
        self.password.set_text(pyneighborhood.userconfig.get("Main", "password"))
        if pyneighborhood.userconfig.get("CIFS", "enable") == "True":
            self.type.insert_text(0, "CIFS")
        if pyneighborhood.userconfig.get("SMB", "enable") == "True":
            self.type.append_text("SMB")
        self.type.set_active(0)
        self.errorre = re.compile("^.* ([0-9]+) = (.*)")

    def logon_toggle(self, widget):
        """
        Logon Options Toggle Callback
        """
        if self.authtoggle.get_active() == True:
            self.user.set_sensitive(True)
            self.password.set_sensitive(True)
        else:
            self.user.set_sensitive(False)
            self.password.set_sensitive(False)

    def apply(self, widget):
        """
        The Apply Button Callback
        """
        self.start()
        self.window.hide_all()

    def abort(self, widget):
        """
        The Abort Button Callback
        """
        self.window.destroy()

    def run(self):
        """
        Make it real!
        """
        mountcmdlist = []
        mountcmdlist.append("gksudo --")#FIXME: Should be a Config Option
        mountcmdlist.append(pyneighborhood.userconfig.get(self.type.get_active_text(), "mount"))
        mountcmdlist.append("//" + self.host.name + "/" + self.share)
        mountcmdlist.append(pyneighborhood.userconfig.get("Main", "mount_directory") + "/"+  self.host.name + "/" + self.share)
        # FIXME: Should be a better Implementation:
        if not os.access(mountcmdlist[3], os.R_OK + os.W_OK):
            try:
                os.mkdir(mountcmdlist[3][:-len("/" + self.share)])
            except OSError:
                pass
            os.mkdir(mountcmdlist[3])
        self.mainwindow.treestore.set_value(self.iter, self.mainwindow.MOUNTPOINT, mountcmdlist[3])
        mountcmdlist.append("-o")
        if self.anontoggle.get_active() == True:
            mountcmdlist.append("guest")
        elif self.authtoggle.get_active() == True:
            mountcmdlist.append("username=%(user)s" % { "user":self.user.get_text() })
            os.environ["PASSWD"] = self.password.get_text()
        f = os.popen(string.join(mountcmdlist, " "))
        error = self.errorre.match(f.read())
        f.close()
        if error != None:
            print error.groups()#FIXME: Should be in the Statusbar
        self.mainwindow.reload_mounts()
        self.window.destroy()
