# nmblookup.py
# Copyright (C) 2008 Stefan J. Betz <stefan_betz@gmx.net>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import pyneighborhood
import string
import socket
import os
import re

class nmblookup(object):
    """
    Parser for the nmblookup Application
    """

    def __init__(self):
        """
        Constructor
        """
        self.iplookupcommand = string.join((pyneighborhood.userconfig.get("Main", "nmblookup"), "-T"), " ")
        self.ipre = re.compile(".*\n(.*), ([0-9\.]*) (.*)<00>")
        self.hostlookupcommand = string.join((pyneighborhood.userconfig.get("Main", "nmblookup"), "-A"), " ")
        self.masterlookupcommand = string.join((pyneighborhood.userconfig.get("Main", "nmblookup"), "-M"), " ")
        self.masterre = re.compile(".*\n([0-9\.]*) (.*)<1d>")
        self.masternetbiosre = re.compile(".*\n\t(.*)<00>.*\n")
        self.workgroupre = re.compile("\t(.*)<00.*GROUP.*")

    def retrieve_ip(self, host):
        """
        Fill out the ip from host
        """
        if host.name != "":
            f = os.popen(self.iplookupcommand + " \"" + host.name + "\"")
            output = f.read()
            f.close()
            match = self.ipre.match(output)
            if match != None:
                host.dns_name = match.group(1)
                host.ip = match.group(2)

    def retrieve_host(self, host):
        """
        Fill out the host from host by IP
        """
        pass

    def retrieve_data(self, host):
        """
        Get hostname and/or ip via nmblookup
        """
        pass

    def retrieve_master(self, workgroup):
        """
        Get Workgroup Master Browser IP and/or DNS Name
        """
        if workgroup.name != "":
            f = os.popen(self.masterlookupcommand + " \"" + workgroup.name + "\"")
            output = f.read()
            f.close()
            match = self.masterre.match(output)
            if match != None:
                workgroup.master_ip = match.group(1)
                workgroup.master = socket.getfqdn(match.group(1))
                f = os.popen(self.hostlookupcommand + " \"" + workgroup.master_ip + "\"")
                output = f.read()
                f.close()
                hostmatch = self.masternetbiosre.match(output)
                if hostmatch != None:
                    workgroup.master_netbios = hostmatch.group(1).strip()

    def retrieve_workgroup(self, host):
        """
        Get the Workgroup to host
        """
        f = os.popen(self.hostlookupcommand + " " + host.ip)
        output = f.read()
        f.close()
        match = self.workgroupre.search(output)
        if match != None:
            host.workgroup = match.group(1).strip()

    def groups(self):
        """
        Get the Workgroups
        """
        pass

    def groups_msbrowse(self):
        """
        Get the Workgroups via msbrowse
        """
        pass
