## options.py
## 
## Copyright (C) 2006-2007 Mykola Lynnyk <pydefiner@gmail.com>
##
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published
## by the Free Software Foundation; version 3 only.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##

import pygtk
pygtk.require('2.0')
import gtk
import gtk.glade
import string

import pyneighborhood
import gettext

_ = gettext.gettext

class OptionsDialog(object):
    """
    Class for the Options Dialog
    """
    def path_validator(self, path):
        """
        Fix the entered Path
        """
        list = path.split("/")
        final_path = ""
        for item in list:
            if item != "":
                final_path = final_path + "/" + item
        return final_path

    def filesystems_handler(self, widget):
        """
        Callback if Filesystems changes...
        """
        if self.smb_toggle.get_active() == True or self.cifs_toggle.get_active() == True:
            pass
        else:
            print "FIXME: At leat one Filesystem must enabled"
        self.smb_mount.set_sensitive(self.smb_toggle.get_active())
        self.smb_umount.set_sensitive(self.smb_toggle.get_active())
        self.smb_options.set_sensitive(self.smb_toggle.get_active())
        self.cifs_mount.set_sensitive(self.cifs_toggle.get_active())
        self.cifs_umount.set_sensitive(self.cifs_toggle.get_active())
        self.cifs_options.set_sensitive(self.cifs_toggle.get_active())

    def response_handler(self, widget, response_id):
        """
        Dialog Callback for the respone...
        """
        if response_id == gtk.RESPONSE_CANCEL:
            self.window.destroy()
            return gtk.FALSE
        elif response_id == gtk.RESPONSE_OK:
            # CIFS Options
            pyneighborhood.userconfig.set("CIFS", "enable", str(self.cifs_toggle.get_active()))
            pyneighborhood.userconfig.set("CIFS", "mount", pyneighborhood.unpad(self.cifs_mount.get_text()))
            pyneighborhood.userconfig.set("CIFS", "options", pyneighborhood.unpad(self.cifs_options.get_text()))
            pyneighborhood.userconfig.set("CIFS", "unmount", pyneighborhood.unpad(self.cifs_umount.get_text()))
            # Main Options
            pyneighborhood.userconfig.set("Main", "cleanup_dirs", str(self.mountpoint_cleanup.get_active()))
            pyneighborhood.userconfig.set("Main", "enable_file_manager", str(self.launch_filemanager.get_active()))
            pyneighborhood.userconfig.set("Main", "filemanager", str(self.filemanager.get_text()))
            pyneighborhood.userconfig.set("Main", "mount_directory", self.path_validator(pyneighborhood.unpad(self.mountpoint.get_text())))
            pyneighborhood.userconfig.set("Main", "nmblookup", pyneighborhood.unpad(self.nmblookup.get_text()))
            pyneighborhood.userconfig.set("Main", "password", pyneighborhood.unpad(self.password.get_text()))
            pyneighborhood.userconfig.set("Main", "smbclient", pyneighborhood.unpad(self.smbclient.get_text()))
            pyneighborhood.userconfig.set("Main", "smbtree", pyneighborhood.unpad(self.smbtree.get_text()))
            pyneighborhood.userconfig.set("Main", "username", pyneighborhood.unpad(self.username.get_text()))
            # SMB Options
            pyneighborhood.userconfig.set("SMB", "enable", str(self.smb_toggle.get_active()))
            pyneighborhood.userconfig.set("SMB", "mount", pyneighborhood.unpad(self.smb_mount.get_text()))
            pyneighborhood.userconfig.set("SMB", "options", pyneighborhood.unpad(self.smb_options.get_text()))
            pyneighborhood.userconfig.set("SMB", "unmount", pyneighborhood.unpad(self.smb_umount.get_text()))
            f = open(pyneighborhood.config.homeconfdir() + "/config", "w")
            pyneighborhood.userconfig.write(f)
            f.flush()
            f.close()
            self.window.destroy()
            return gtk.TRUE

    def launch_filemanager_toggle(self, widget):
        """
        Enables or Disables the Filemanager Entry
        """
        if self.launch_filemanager.get_active() == True:
            self.filemanager.set_sensitive(True)
        else:
            self.filemanager.set_sensitive(False)

    def __init__(self, parent):
        """
        Constructor, creates the Windows and many more...
        """
        self.parent = parent
        self.xml = gtk.glade.XML(pyneighborhood.config.sharepath() + "/pyNeighborhood.glade", "options")
        self.window = self.xml.get_widget("options")
        # Automatic Signal->Callback Connection:
        self.xml.signal_autoconnect(self)
        # SMB Options
        self.smb_toggle = self.xml.get_widget("smb")
        self.smb_mount = self.xml.get_widget("smbmount")
        self.smb_mount.set_text(pyneighborhood.userconfig.get("SMB", "mount"))
        self.smb_umount = self.xml.get_widget("smbumount")
        self.smb_umount.set_text(pyneighborhood.userconfig.get("SMB", "unmount"))
        self.smb_options = self.xml.get_widget("smboptions")
        self.smb_options.set_text(pyneighborhood.userconfig.get("SMB", "options"))
        # CIFS Options
        self.cifs_toggle = self.xml.get_widget("cifs")
        self.cifs_mount = self.xml.get_widget("cifsmount")
        self.cifs_mount.set_text(pyneighborhood.userconfig.get("CIFS", "mount"))
        self.cifs_umount = self.xml.get_widget("cifsumount")
        self.cifs_umount.set_text(pyneighborhood.userconfig.get("CIFS", "unmount"))
        self.cifs_options = self.xml.get_widget("cifsoptions")
        self.cifs_options.set_text(pyneighborhood.userconfig.get("CIFS", "options"))
        # SMB Activation:
        if pyneighborhood.userconfig.get("SMB", "enable") == "True":
            self.smb_toggle.set_active(True)
        else:
            self.smb_toggle.set_active(False)
        # CIFS Activation:
        if pyneighborhood.userconfig.get("CIFS", "enable") == "True":
            self.cifs_toggle.set_active(True)
        else:
            self.cifs_toggle.set_active(False)
        # Mounpoint Options
        self.mountpoint = self.xml.get_widget("mountpoint-entry")
        self.mountpoint.set_text(pyneighborhood.userconfig.get("Main", "mount_directory"))
        self.mountpoint_cleanup = self.xml.get_widget("mountpoint-cleanup")
        if pyneighborhood.userconfig.get("Main", "cleanup_dirs") == "True":
            self.mountpoint_cleanup.set_active(True)
        else:
            self.mountpoint_cleanup.set_active(False)
        # Login Options
        self.username = self.xml.get_widget("username-entry")
        self.username.set_text(pyneighborhood.userconfig.get("Main", "username"))
        self.password = self.xml.get_widget("password-entry")
        self.password.set_text(pyneighborhood.userconfig.get("Main", "password"))
        # Browsing Options
        self.nmblookup = self.xml.get_widget("nmblookup-entry")
        self.nmblookup.set_text(pyneighborhood.userconfig.get("Main", "nmblookup"))
        self.smbclient = self.xml.get_widget("smbclient-entry")
        self.smbclient.set_text(pyneighborhood.userconfig.get("Main", "smbclient"))
        self.smbtree = self.xml.get_widget("smbtree-entry")
        self.smbtree.set_text(pyneighborhood.userconfig.get("Main", "smbtree"))
        # Filemanager Options
        self.launch_filemanager = self.xml.get_widget("options-launch-filemanager")
        self.filemanager = self.xml.get_widget("options-filemanager-entry")
        self.filemanager.set_text(pyneighborhood.userconfig.get("Main", "filemanager"))
        if pyneighborhood.userconfig.get("Main", "enable_file_manager") == "True":
            self.launch_filemanager.set_active(True)
            self.filemanager.set_sensitive(True)
        else:
            self.launch_filemanager.set_active(False)
            self.filemanager.set_sensitive(False)
